#pragma once

#include "LibLsp/JsonRpc/MessageJsonHandler.h"
namespace lsp {
        class ProtocolJsonHandler : public MessageJsonHandler
        {
        public:
                ProtocolJsonHandler();
        };

}
