#!/usr/bin/env python3
# Enum class for v3dtypes
""" AUTO-GENERATED from v3dtypes.csv """
# Generated at 2023-02-09 04:54:07

class v3dtypes:
    v3dtypes_material=1
    v3dtypes_transform=2
    v3dtypes_element=3
    v3dtypes_centers=4
    v3dtypes_header=5
    v3dtypes_line=64
    v3dtypes_triangle=65
    v3dtypes_quad=66
    v3dtypes_curve=128
    v3dtypes_bezierTriangle=129
    v3dtypes_bezierPatch=130
    v3dtypes_lineColor=192
    v3dtypes_triangleColor=193
    v3dtypes_quadColor=194
    v3dtypes_curveColor=256
    v3dtypes_bezierTriangleColor=257
    v3dtypes_bezierPatchColor=258
    v3dtypes_triangles=512
    v3dtypes_disk=1024
    v3dtypes_cylinder=1025
    v3dtypes_tube=1026
    v3dtypes_sphere=1027
    v3dtypes_halfSphere=1028
    v3dtypes_animation=2048
    v3dtypes_pixel=4096
# End of File
