// Enum class for v3dtypes
// AUTO-GENERATED from v3dtypes.csv
// Generated at 2023-02-09 04:54:00

namespace camp
{
enum v3dtypes : uint32_t
{
material=1,
transform=2,
element=3,
centers=4,
header=5,
line=64,
triangle=65,
quad=66,
curve=128,
bezierTriangle=129,
bezierPatch=130,
lineColor=192,
triangleColor=193,
quadColor=194,
curveColor=256,
bezierTriangleColor=257,
bezierPatchColor=258,
triangles=512,
disk=1024,
cylinder=1025,
tube=1026,
sphere=1027,
halfSphere=1028,
animation=2048,
pixel=4096,
};

} // namespace camp
// End of File
